/*
 * PPF c++ wrapper
 *
 * Copyright (C) 2008 Alex Meakins
 *
 * 2008-04-09: Alex Meakins <alex.meakins@jet.uk>
 *             initial version
 *
 * This file is part of PPFc++.
 *
 * PPFc++ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PPFc++ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser Public License for more details.
 *
 * You should have received a copy of the GNU Lesser Public License
 * along with PPFc++.  If not, see <http://www.gnu.org/licenses/>.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __PPFTYPES_HPP__
#define __PPFTYPES_HPP__

using namespace std;

namespace ppf
{
	// enumerations and constants ----------------------------------------------

	/// Access flags are used in conjunction with the connect() method to specify read and write access to the PPF system.
	typedef int accessflag;
	static const accessflag read=0x01;      ///< Open PPF system for reading.
	static const accessflag write=0x02;     ///< Open PPF system for writing.

	/// Process flags can be used to modify the operation of a number of data get and set methods.
	enum processflag
	{
		nearest,	///< Use nearest indexed value.
		linear,		///< Use linear interpolation.
		cubic,		///< Use cubic interpolation.
		preserve,	///< No not modify data.
	};

	/// I/O flags can be used to modify the reading and writing of PPF DDA datatypes.
	typedef int ioflag;
	static const ioflag tvector=0x01;		///< Enables reading or writing of the T vector.
	static const ioflag xvector=0x02;     		///< Enables reading or writing of the X vector.
	static const ioflag noxvector=0x04;			///< Disables reading or writing of the X vector.
	static const ioflag notvector=0x08;			///< Disables reading or writing of the T vector.
	static const ioflag lastxvector=0x10;			///< (Writing only) Instructs the PPF system to reuse the X vector of a previously written datatype.
	static const ioflag lasttvector=0x20;			///< (Writing only) Instructs the PPF system to reuse the T vector of a previously written datatype.

	// classes -----------------------------------------------------------------

	/// A concise container for an (x,t) co-ordinate and corresponding data value.
	class coord
	{
		public:
		// variables
			float t;    ///< t coordinate
			float x;    ///< x coordinate
			float d;	///< value at(\e t, \e x)

		// constructor
		
		    // Basic constructor.
		    coord() : t(0), x(0), d(0) {};

	};

	/// PPF system exception class.
	/// Thrown when an error occurs in the PPF system.
	class ppf_error
	{
		public:
		// variables
			const int code;           ///< Holds the PPF system error code.
			const string method;      ///< Holds the name of PPF method which failed.
			const string message;     ///< Holds a decriptive error message.

		// constructors/destructors

			// basic constructors ----------------------------------------------

			/// Basic Constructor.
			ppf_error() : method(""), code(0), message("An error occurred accessing the PPF system.") {};

			// initiating constructors -----------------------------------------------

			/// Initialising constructor.
			/// Sets error message to passed string. PPF routine and error code are set to null.
			/// \param errormsg String containing the error message.
			ppf_error(string errormsg) : method(""), code(0), message(errormsg) {};

			/// Initialising constructor.
			/// Sets PPF routine and error code to passed values. Sets message to a default error message.
			/// \param name String containing name of PPF routine which failed.
			/// \param value Integer containing error code returned by failed PPF routine.
			ppf_error(string name, int value) : method(name), code(value), message("An error occurred accessing the PPF system.") {};

			/// Initialising constructor.
			/// Sets failed ppf routine name, code and error message to passed values.
			/// \param name String containing name of PPF routine which failed.
			/// \param value Integer containing error code returned by failed PPF routine.
			/// \param errormsg String containing the error message.
			ppf_error(string name, int value, string errormsg) : method(name), code(value), message(errormsg) {};
	};

};

#endif
