/*
 * PPF c++ wrapper
 *
 * Copyright (C) 2008 Alex Meakins
 *
 * 2008-04-09: Alex Meakins <alex.meakins@jet.uk>
 *             initial version
 *
 * This file is part of PPFc++.
 *
 * PPFc++ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PPFc++ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser Public License for more details.
 *
 * You should have received a copy of the GNU Lesser Public License
 * along with PPFc++.  If not, see <http://www.gnu.org/licenses/>.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __PPFSYSTEM_HPP__
#define __PPFSYSTEM_HPP__

#include <vector>
#include <string>
#include "ppftypes.hpp"
#include "ppfdatatype.hpp"

using namespace std;

namespace ppf
{
	/// The main PPF system class through which all PPF access is performed.
	class system
	{
		public:

		// methods -------------------------------------------------------------

  			/// Opens a connection to the public (user = "JETPPF") PPF system for reading only.
  			/// To use an alternate user name or open a write connection one of the alternate versions of this routine, connect(const string user) or connect(const string user, const accessflag flag), must be used.
			void connect() const;

  			/// Opens a connection to the PPF system using the supplied user name for reading only.
  			/// To open a write connection the full version this routine, connect(const string user, const accessflag flag), must be used.
			/// \param user A string containing the user name to connect with.
			void connect(string user) const;

  			/// Opens a connection to the PPF system using the supplied parameters.
			/// The access flags are used to determine if the connection is for read access, write access or both.
			/// Valid access flags are: \e read, \e write
			/// \param user String containing the user name to connect with.
			/// \param flag List of access flags.
			void connect(string user, const accessflag flag) const;

			/// Reads in data for the given shot, DDA and dataype.
			/// \param d Datatype object to write data to
			/// \param shot Shot number
			/// \param DDA String containing the (up to) 4 character DDA name
			/// \param dtype String containing the (up to) 4 character datatype name
			/// \exception ppf_error thrown if a read error occurs, error details are returned in ppf_error
			void read(datatype &d, const long shot, string DDA, string dtype) const;

			/// Reads in data for the given shot, DDA and dataype using I/O flags.
			/// I/O flags modify the loading of the datatype.
			/// \param d Datatype object to write data to
			/// \param shot Shot number
			/// \param DDA String containing the (up to) 4 character DDA name
			/// \param dtype String containing the (up to) 4 character datatype name
			/// \param flag List of I/O flags
			/// \exception ppf_error thrown if a read error occurs, error details are returned in ppf_error
			void read(datatype &d, const long shot, string DDA, string dtype, const ioflag flag) const;

			/// Reads in data for the given shot, sequence, DDA and dataype.
			/// \param d Datatype object to write data to
			/// \param shot Shot number
			/// \param sequence Sequence number
			/// \param DDA String containing the (up to) 4 character DDA name
			/// \param dtype String containing the (up to) 4 character datatype name
			/// \exception ppf_error thrown if a read error occurs, error details are returned in ppf_error
			void read(datatype &d, const long shot, const long sequence, string DDA, string dtype) const;

			/// Reads in data for the given shot, sequence, DDA and dataype using I/O flags.
			/// I/O flags modify the loading of the datatype.
			/// \param d Datatype object to write data to
			/// \param shot Shot number
			/// \param sequence Sequence number
			/// \param DDA String containing the (up to) 4 character DDA name
			/// \param dtype String containing the (up to) 4 character datatype name
			/// \param flag List of I/O flags
			/// \exception ppf_error thrown if a read error occurs, error details are returned in ppf_error
			void read(datatype &d, const long shot, const long sequence, string DDA, string dtype, const ioflag flag) const;

			/// Creates a new temporary PPF for writing.
			/// Write access is required to create a new PPF. The new PPF will not be added to the system until at least one datatype is written to it and commit() has been called.
			/// \param shot Shot Number
			/// \exception ppf_error thrown if an error occurs, error details are returned in ppf_error
			void create(const long shot);

			/// Creates a new temporary PPF for writing.
			/// Write access is required to create a new PPF. A status flag for the PPF can be provided.
			///	The new PPF will not be added to the system until at least one datatype is written to it and commit() has been called.
			/// \param shot Shot Number
			/// \param status PPF system status flag
			/// \exception ppf_error thrown if an error occurs, error details are returned in ppf_error
			void create(const long shot, const long status);

			/// Creates a new temporary PPF for writing.
			/// Write access is required to create a new PPF. A descriptive text comment for the PPF can be provided.
			///	The new PPF will not be added to the system until at least one datatype is written to it and commit() has been called.
			/// \param shot Shot Number
			/// \param comment String holding a comment
			/// \exception ppf_error thrown if an error occurs, error details are returned in ppf_error
			void create(const long shot, const string comment);

			/// Creates a new temporary PPF for writing.
			/// Write access is required to create a new PPF. A descriptive text comment and status flag for the PPF can be provided.
			///	The new PPF will not be added to the system until at least one datatype is written to it and commit() has been called.
			/// \param shot Shot Number
			/// \param status PPF system status flag
			/// \param comment String holding a comment
			/// \exception ppf_error thrown if an error occurs, error details are returned in ppf_error
			void create(const long shot, const long status, const string comment);

			/// Writes the provided datatype to an open PPF.
			/// Writing can only be performed provided a new temporary PPF has been previously opened via a call to create(long shot).
			/// \param d Datatype object to write to PPF
			/// \param DDA String containing the (up to) 4 character DDA name
			/// \param dtype String containing the (up to) 4 character datatype name
			/// \exception ppf_error thrown if a write error occurs, error details are returned in ppf_error
			void write(datatype &d, string DDA, string dtype);

			/// Writes the provided datatype to an open PPF using I/O flags.
			/// Writing can only be performed provided a new temporary PPF has been previously opened via a call to create(long shot).
			/// \param d Datatype object to write to PPF
			/// \param DDA String containing the (up to) 4 character DDA name
			/// \param dtype String containing the (up to) 4 character datatype name
			/// \param flag List of I/O flags
			/// \exception ppf_error thrown if a write error occurs, error details are returned in ppf_error
			void write(datatype &d, string DDA, string dtype, const ioflag flag);
			
			/// Closes DDA to futher writing.
			/// \exception ppf_error thrown if the DDA can not be closed properly, error details are returned in ppf_error
			void closeDDA() const;

			/// Closes DDA to futher writing.
			/// A DDA status flag can be provided which will be written to the DDA.
			/// \param status DDA status flag
			/// \exception ppf_error thrown if the DDA can not be closed properly, error details are returned in ppf_error
			void closeDDA(const int status) const;

			/// Closes DDA to futher writing.
			/// A comment can be provided which will be written to the DDA.
			/// \param comment String holding a comment
			/// \exception ppf_error thrown if the DDA can not be closed properly, error details are returned in ppf_error
			void closeDDA(string comment) const;
			
			/// Closes DDA to futher writing.
			/// A comment and DDA status flag can be provided which will be written to the DDA.
			/// \param status DDA status flag
			/// \param comment String holding a comment
			/// \exception ppf_error thrown if the DDA can not be closed properly, error details are returned in ppf_error
			void closeDDA(const int status, string comment) const;
			
			/// Deletes the temporary PPF without writing it to the PPF system.
			/// All data written to the temporary PPF will be lost.
			/// \exception ppf_error thrown if there is a problem, error details are returned in ppf_error
			void abort() const;
			
			/// Commits a temporary PPF to the PPF system.
			/// The temporary PPF will be finalised and written to the PPF system.
			/// \exception ppf_error thrown if the temporary PPF can not be written, error details are returned in ppf_error
			int commit() const;

			/// Commits a temporary PPF to the PPF system.
			/// The temporary PPF will be finalised and written to the PPF system. A string identifying the program used to create the PPF and a version number may be supplied.
			/// \param program String containing the program name (max. 8 characters)
			/// \param version Program version number
			/// \exception ppf_error thrown if the temporary PPF can not be written, error details are returned in ppf_error
			int commit(const string program, const int version) const;

		private:

		// variables
		
		    long openshot;	///< Holds the shot number of the PPF opened for writing.
   		    long lastXRef;	///< Holds the most recent X vector reference returned in the last datatype write.
		    long lastTRef;	///< Holds the most recent T vector reference returned in the last datatype write.
	};
};

#endif

